@@ -19,13 +19,15 @@ struct ClockInfo {
 }
 #[cfg(not(target_os = "macos"))]
 fn main() {
-    let ctl = "kern.clockrate";
-    println!("\nRead sysctl {} and parse result to struct ClockInfo", ctl);
+    let ctl = sysctl::Ctl::new("kern.clockrate").expect("could not get sysctl: kern.clockrate");
 
-    let d = sysctl::description(ctl).unwrap();
+    let name = ctl.name().expect("could not get sysctl name");
+    println!("Read sysctl {} and parse result to struct ClockInfo", name);
+
+    let d = ctl.description().expect("could not get sysctl description");
     println!("Description: {:?}", d);
 
-    let val_enum = sysctl::value(ctl).unwrap();
+    let val_enum = ctl.value().expect("could not get sysctl value");
     println!("ClockInfo raw data: {:?}", val_enum);
 
     if let sysctl::CtlValue::Struct(val) = val_enum {
