@@ -7,23 +7,32 @@
 //! extern crate sysctl;
 //! #[cfg(not(target_os = "macos"))]
 //! fn main() {
+//!     let ctl = sysctl::Ctl::new("kern.osrevision")
+//!         .expect("could not get sysctl");
 //!
-//!     let ctl = "kern.osrevision";
+//!     let d = ctl.description()
+//!         .expect("could not get description");
 //!
-//!     let d: String = sysctl::description(ctl).unwrap();
 //!     println!("Description: {:?}", d);
 //!
-//!     let val_enum = sysctl::value(ctl).unwrap();
+//!     let val_enum = ctl.value()
+//!         .expect("could not get value");
+//!
 //!     if let sysctl::CtlValue::Int(val) = val_enum {
 //!         println!("Value: {}", val);
 //!     }
 //! }
+//!
 //! #[cfg(target_os = "macos")]
 //! fn main() {
+//!     let mut ctl = sysctl::Ctl::new("kern.osrevision")
+//!         .expect("could not get sysctl");
 //!
-//!     let ctl = "kern.osrevision";
+//!     // description is not available on macos
+//!
+//!     let val_enum = ctl.value()
+//!         .expect("could not get value");
 //!
-//!     let val_enum = sysctl::value(ctl).unwrap();
 //!     if let sysctl::CtlValue::Int(val) = val_enum {
 //!         println!("Value: {}", val);
 //!     }
@@ -59,20 +68,20 @@ extern crate libc;
 #[macro_use]
 extern crate failure;
 
-use libc::{c_int, c_uchar, c_uint, c_void};
 use libc::sysctl;
 use libc::BUFSIZ;
+use libc::{c_int, c_uchar, c_uint, c_void};
 
+use byteorder::{ByteOrder, LittleEndian, WriteBytesExt};
 use std::cmp;
 use std::convert;
+#[cfg(not(target_os = "macos"))]
+use std::f32;
 use std::io;
 use std::mem;
 use std::ptr;
 use std::str;
 use std::str::FromStr;
-#[cfg(not(target_os = "macos"))]
-use std::f32;
-use byteorder::{ByteOrder, LittleEndian, WriteBytesExt};
 
 // CTL* constants belong to libc crate but have not been added there yet.
 // They will be removed from here once in the libc crate.
@@ -320,20 +329,66 @@ pub enum CtlValue {
 }
 
 #[derive(Debug, PartialEq)]
-struct CtlInfo {
-    ctl_type: CtlType,
-    fmt: String,
+/// A structure representing control metadata
+pub struct CtlInfo {
+    /// The control type.
+    pub ctl_type: CtlType,
+
+    /// A string which specifies the format of the OID in
+    /// a symbolic way.
+    ///
+    /// This format is used as a hint by sysctl(8) to
+    /// apply proper data formatting for display purposes.
+    ///
+    /// Formats defined in sysctl(9):
+    /// * `N`       node
+    /// * `A`       char *
+    /// * `I`       int
+    /// * `IK[n]`   temperature in Kelvin, multiplied by an optional single
+    ///    digit power of ten scaling factor: 1 (default) gives deciKelvin,
+    ///    0 gives Kelvin, 3 gives milliKelvin
+    /// * `IU`      unsigned int
+    /// * `L`       long
+    /// * `LU`      unsigned long
+    /// * `Q`       quad_t
+    /// * `QU`      u_quad_t
+    /// * `S,TYPE`  struct TYPE structures
+    pub fmt: String,
+
     flags: u32,
 }
-#[cfg(not(target_os = "macos"))]
+
 impl CtlInfo {
-    fn is_temperature(&self) -> bool {
+    /// Return the flags for this sysctl.
+    pub fn flags(&self) -> CtlFlags {
+        CtlFlags::from_bits_truncate(self.flags)
+    }
+
+    /// Is this sysctl a temperature?
+    #[cfg(not(target_os = "macos"))]
+    pub fn is_temperature(&self) -> bool {
         self.fmt.starts_with("IK")
     }
+
+    /// If the sysctl is a structure, return the structure type string.
+    ///
+    /// Checks whether the format string starts with `S,` and returns the rest
+    /// of the format string or None if the format String does not have a struct
+    /// hint.
+    pub fn struct_type(&self) -> Option<String> {
+        if !self.fmt.starts_with("S,") {
+            return None;
+        }
+
+        Some(self.fmt[2..].into())
+    }
 }
 
 #[derive(Debug, Fail)]
 pub enum SysctlError {
+    #[fail(display = "no such sysctl: {}", _0)]
+    NotFound(String),
+
     #[fail(display = "no matching type for value")]
     #[cfg(not(target_os = "macos"))]
     UnknownType,
@@ -342,16 +397,10 @@ pub enum SysctlError {
     ExtractionError,
 
     #[fail(display = "IO Error: {}", _0)]
-    IoError(
-        #[cause]
-        io::Error
-    ),
+    IoError(#[cause] io::Error),
 
     #[fail(display = "Error parsing UTF-8 data: {}", _0)]
-    Utf8Error(
-        #[cause]
-        str::Utf8Error
-    ),
+    Utf8Error(#[cause] str::Utf8Error),
 
     #[fail(display = "Value is not readable")]
     NoReadAccess,
@@ -359,8 +408,15 @@ pub enum SysctlError {
     #[fail(display = "Value is not writeable")]
     NoWriteAccess,
 
-    #[fail(display = "sysctl returned a short read: read {} bytes, while a size of {} was reported", read, reported)]
+    #[fail(
+        display = "sysctl returned a short read: read {} bytes, while a size of {} was reported",
+        read,
+        reported
+    )]
     ShortRead { read: usize, reported: usize },
+
+    #[fail(display = "Error reading C String: String was not NUL-terminated.")]
+    InvalidCStr(#[cause] std::ffi::FromBytesWithNulError),
 }
 
 /// A custom type for temperature sysctls.
@@ -370,8 +426,14 @@ pub enum SysctlError {
 /// extern crate sysctl;
 /// #[cfg(not(target_os = "macos"))]
 /// fn main() {
-///     let val_enum = sysctl::value("dev.cpu.0.temperature").unwrap();
-///     if let sysctl::CtlValue::Temperature(val) = val_enum {
+/// #   let ctl = match sysctl::Ctl::new("dev.cpu.0.temperature") {
+/// #       Ok(c) => c,
+/// #       Err(e) => {
+/// #           println!("Couldn't get dev.cpu.0.temperature: {}", e);
+/// #           return;
+/// #       }
+/// #   };
+///     if let Ok(sysctl::CtlValue::Temperature(val)) = ctl.value() {
 ///         println!("Temperature: {:.2}K, {:.2}F, {:.2}C",
 ///                  val.kelvin(),
 ///                  val.fahrenheit(),
@@ -421,7 +483,11 @@ fn name2oid(name: &str) -> Result<Vec<c_int>, SysctlError> {
         )
     };
     if ret < 0 {
-        return Err(SysctlError::IoError(io::Error::last_os_error()));
+        let e = io::Error::last_os_error();
+        return Err(match e.kind() {
+            std::io::ErrorKind::NotFound => SysctlError::NotFound(name.into()),
+            _ => SysctlError::IoError(e),
+        });
     }
 
     // len is in bytes, convert to number of c_ints
@@ -496,10 +562,11 @@ fn oidfmt(oid: &[c_int]) -> Result<CtlInfo, SysctlError> {
     let ctltype_val = kind & CTLTYPE as u32;
 
     // 'fmt' is after 'Kind' in result buffer
-    let fmt: String = match str::from_utf8(&buf[mem::size_of::<u32>()..buf_len]) {
-        Ok(x) => x.to_owned(),
-        Err(e) => return Err(SysctlError::Utf8Error(e)),
-    };
+    let fmt: String =
+        match std::ffi::CStr::from_bytes_with_nul(&buf[mem::size_of::<u32>()..buf_len]) {
+            Ok(x) => x.to_string_lossy().into(),
+            Err(e) => return Err(SysctlError::InvalidCStr(e)),
+        };
 
     let s = CtlInfo {
         ctl_type: CtlType::from(ctltype_val),
@@ -1060,9 +1127,17 @@ pub fn value_oid_as<T>(oid: &mut Vec<i32>) -> Result<Box<T>, SysctlError> {
 /// # Example
 /// ```
 /// extern crate sysctl;
+/// # extern crate libc;
 ///
 /// fn main() {
-///     println!("{:?}", sysctl::set_value("hw.usb.debug", sysctl::CtlValue::Int(1)));
+/// #   if unsafe { libc::getuid() } == 0 {
+/// #   let old_value = sysctl::value("hw.usb.debug").unwrap();
+///     let new_value = sysctl::set_value("hw.usb.debug", sysctl::CtlValue::Int(1))
+///         .expect("could not set sysctl value");
+///     assert_eq!(new_value, sysctl::CtlValue::Int(1));
+/// #   // restore old value
+/// #   sysctl::set_value("hw.usb.debug", old_value);
+/// #   } // getuid() == 0
 /// }
 /// ```
 #[cfg(not(target_os = "macos"))]
@@ -1398,7 +1473,7 @@ pub fn next_oid(oid: &Vec<c_int>) -> Result<Option<Vec<c_int>>, SysctlError> {
 /// This struct represents a system control.
 #[derive(Debug, Clone, PartialEq)]
 pub struct Ctl {
-    oid: Vec<c_int>,
+    pub oid: Vec<c_int>,
 }
 
 impl FromStr for Ctl {
@@ -1416,6 +1491,9 @@ impl Ctl {
     ///
     /// This is just a wrapper around `Ctl::from_str`.
     ///
+    /// Returns a result containing the struct Ctl on success or a SysctlError
+    /// on failure.
+    ///
     /// # Example
     ///
     /// ```
@@ -1424,6 +1502,19 @@ impl Ctl {
     ///
     /// let ctl = Ctl::new("kern.osrelease");
     /// ```
+    ///
+    /// If the sysctl does not exist, `Err(SysctlError::NotFound)` is returned.
+    /// ```
+    /// extern crate sysctl;
+    /// use sysctl::{Ctl, SysctlError};
+    ///
+    /// let ctl = Ctl::new("this.sysctl.does.not.exist");
+    /// match ctl {
+    ///     Err(SysctlError::NotFound(_)) => (),
+    ///     Err(_) => panic!("Wrong error type returned"),
+    ///     Ok(_) => panic!("Nonexistent sysctl seems to exist"),
+    /// }
+    /// ```
     pub fn new(name: &str) -> Result<Self, SysctlError> {
         Ctl::from_str(name)
     }
@@ -1434,8 +1525,8 @@ impl Ctl {
     /// # Example
     ///
     /// ```
-    /// extern crate sysctl;
-    /// use sysctl::Ctl;
+    /// # extern crate sysctl;
+    /// # use sysctl::Ctl;
     /// let ctl = Ctl::new("kern.osrelease").expect("could not get sysctl");
     /// assert_eq!(ctl.name().expect("could not get name"), "kern.osrelease");
     /// ```
@@ -1449,13 +1540,12 @@ impl Ctl {
     /// # Example
     ///
     /// ```
-    /// extern crate sysctl;
-    /// use sysctl::{Ctl, CtlType};
-    ///
-    /// let osrelease = Ctl::new("kern.osrelease")
+    /// # extern crate sysctl;
+    /// # use sysctl::{Ctl, CtlType};
+    /// let ctl = Ctl::new("kern.osrelease")
     ///     .expect("Could not get kern.osrelease sysctl");
-    /// let value_type = osrelease.value_type()
-    ///         .expect("Could notget kern.osrelease value type");
+    /// let value_type = ctl.value_type()
+    ///         .expect("Could not get kern.osrelease value type");
     /// assert_eq!(value_type, CtlType::String);
     /// ```
     pub fn value_type(self: &Self) -> Result<CtlType, SysctlError> {
@@ -1593,18 +1683,20 @@ impl Ctl {
     /// ```
     /// extern crate sysctl;
     /// use sysctl::Ctl;
+    /// # extern crate libc;
     ///
     /// fn main() {
+    /// # if unsafe { libc::getuid() } == 0 {
     ///     let usbdebug = Ctl::new("hw.usb.debug")
     ///         .expect("could not get hw.usb.debug control");
-    ///     let original = usbdebug.value()
-    ///         .expect("could not get value");
-    ///     let set = usbdebug.set_value(sysctl::CtlValue::Int(0));
-    ///     println!("hw.usb.debug: {:?} -> {:?}", original, set);
-    ///     let set = usbdebug.set_value(sysctl::CtlValue::Int(1));
-    ///     println!("hw.usb.debug: 0 -> {:?}", set);
-    ///     let reset = usbdebug.set_value(original);
-    ///     println!("hw.usb.debug: 1 -> {:?}", reset);
+    /// #   let original = usbdebug.value()
+    /// #       .expect("could not get value");
+    ///     let set = usbdebug.set_value(sysctl::CtlValue::Int(1))
+    ///         .expect("could not set value");
+    ///     assert_eq!(set, sysctl::CtlValue::Int(1));
+    ///     println!("hw.usb.debug: -> {:?}", set);
+    /// #   usbdebug.set_value(original).unwrap();
+    /// # } // getuid() == 0
     /// }
     #[cfg(not(target_os = "macos"))]
     pub fn set_value(self: &Self, value: CtlValue) -> Result<CtlValue, SysctlError> {
@@ -1622,14 +1714,11 @@ impl Ctl {
     /// fn main() {
     ///     let usbdebug = Ctl::new("hw.usb.debug")
     ///         .expect("could not get hw.usb.debug control");
-    ///     let original = usbdebug.value()
-    ///         .expect("could not get value");
-    ///     let set = usbdebug.set_value(sysctl::CtlValue::Int(0));
-    ///     println!("hw.usb.debug: {:?} -> {:?}", original, set);
+    /// #   let original = usbdebug.value()
+    /// #       .expect("could not get value");
     ///     let set = usbdebug.set_value(sysctl::CtlValue::Int(1));
-    ///     println!("hw.usb.debug: 0 -> {:?}", set);
-    ///     let reset = usbdebug.set_value(original);
-    ///     println!("hw.usb.debug: 1 -> {:?}", reset);
+    ///     println!("hw.usb.debug: -> {:?}", set);
+    /// #   usbdebug.set_value(original).unwrap();
     /// }
     #[cfg(target_os = "macos")]
     pub fn set_value(self: &Self, value: CtlValue) -> Result<CtlValue, SysctlError> {
@@ -1659,8 +1748,32 @@ impl Ctl {
     /// }
     /// ```
     pub fn flags(self: &Self) -> Result<CtlFlags, SysctlError> {
-        let info: CtlInfo = oidfmt(&self.oid)?;
-        Ok(CtlFlags::from_bits_truncate(info.flags))
+        Ok(self.info()?.flags())
+    }
+
+    /// Returns a Result containing the control metadata for a sysctl.
+    ///
+    /// Returns a Result containing the CtlInfo struct on success,
+    /// or a SysctlError on failure.
+    ///
+    /// # Example
+    /// ```
+    /// extern crate sysctl;
+    /// use sysctl::{Ctl, CtlInfo};
+    ///
+    /// fn main() {
+    ///     let osrev = Ctl::new("kern.osrevision")
+    ///         .expect("could not get control");
+    ///
+    ///     let info = osrev.info()
+    ///         .expect("could not get info");
+    ///
+    ///     // kern.osrevision is not a structure.
+    ///     assert_eq!(info.struct_type(), None);
+    /// }
+    /// ```
+    pub fn info(self: &Self) -> Result<CtlInfo, SysctlError> {
+        Ok(oidfmt(&self.oid)?)
     }
 }
 
@@ -1761,7 +1874,8 @@ mod tests {
         assert_eq!(name, "kern.osrevision");
 
         let ctl = Ctl { oid };
-        let name = ctl.name()
+        let name = ctl
+            .name()
             .expect("Could not get name of kern.osrevision sysctl.");
         assert_eq!(name, "kern.osrevision");
     }
@@ -1867,6 +1981,24 @@ mod tests {
         assert_eq!(s.trim(), ver.trim());
     }
 
+    #[test]
+    fn ctl_struct_type() {
+        let info = CtlInfo {
+            ctl_type: CtlType::Int,
+            fmt: "S,TYPE".into(),
+            flags: 0,
+        };
+
+        assert_eq!(info.struct_type(), Some("TYPE".into()));
+
+        let info = CtlInfo {
+            ctl_type: CtlType::Int,
+            fmt: "I".into(),
+            flags: 0,
+        };
+        assert_eq!(info.struct_type(), None);
+    }
+
     #[test]
     #[cfg(not(target_os = "macos"))]
     fn ctl_description() {
